package com.formos.tapestry.xpath;

import java.util.Iterator;

import org.apache.tapestry5.dom.Attribute;
import org.apache.tapestry5.dom.Comment;
import org.apache.tapestry5.dom.Document;
import org.apache.tapestry5.dom.Element;
import org.apache.tapestry5.dom.Node;
import org.apache.tapestry5.dom.Text;
import org.jaxen.DefaultNavigator;
import org.jaxen.JaxenConstants;
import org.jaxen.UnsupportedAxisException;
import org.jaxen.XPath;
import org.jaxen.util.SingleObjectIterator;



/**
 * Adapts the Tapestry DOM to the Jaxen Navigator API.
 * 
 * @author Paul Field
 */
class TapestryNavigator extends DefaultNavigator {
    private static final long serialVersionUID = 3764663905419316699L;

    public static final TapestryNavigator TAPESTRY_NAVIGATOR = new TapestryNavigator();
    
	private static final int COMMENT_PREFIX_LENGTH = "<!--".length();
	private static final int COMMENT_SUFFIX_LENGTH = "-->".length();
	

    @Override
    public Iterator<?> getAttributeAxisIterator(Object contextNode) {
        if (contextNode instanceof Element) {
            return ((Element)contextNode).getAttributes().iterator();
        }
        return JaxenConstants.EMPTY_ITERATOR;
    }


    @Override
    public Iterator<?> getChildAxisIterator(Object contextNode) {
        if (contextNode instanceof Document) {
            return new SingleObjectIterator(((Document)contextNode).getRootElement());
        } else if (contextNode instanceof Element) {
            return ((Element)contextNode).getChildren().iterator();
        } else {
            return JaxenConstants.EMPTY_ITERATOR;
        }
    }
    
    
    @Override
    public Iterator<?> getParentAxisIterator(Object contextNode) throws UnsupportedAxisException {
    	if (contextNode instanceof Element) {
    		return new SingleObjectIterator(((Element)contextNode).getContainer());    		
    	} else {
    		return super.getParentAxisIterator(contextNode);
    	}
    }

    @Override
    public Object getElementById(Object contextNode, String elementId) {
        return ((Node)contextNode).getDocument().getElementById(elementId);
    }


    @Override
    public Object getDocumentNode(Object contextNode) {
        return ((Node)contextNode).getDocument();
    }


    public String getAttributeName(Object attr) {
        return ((Attribute)attr).getName();
    }

    public String getAttributeNamespaceUri(Object arg0) {
        return null;
    }

    public String getAttributeQName(Object arg0) {
        return null;
    }

    public String getAttributeStringValue(Object attr) {
        return ((Attribute)attr).getValue();
    }

    public String getCommentStringValue(Object comment) {
        String fullComment = ((Comment)comment).toString();
        return fullComment.substring(COMMENT_PREFIX_LENGTH, fullComment.length() - COMMENT_SUFFIX_LENGTH);
    }

    public String getElementName(Object element) {
        return ((Element)element).getName();
    }

    public String getElementNamespaceUri(Object arg0) {
        return null;
    }

    public String getElementQName(Object arg0) {
        return null;
    }

    public String getElementStringValue(Object element) {
        return ((Element)element).getChildMarkup();
    }

    public String getNamespacePrefix(Object arg0) {
        return null;
    }

    public String getNamespaceStringValue(Object arg0) {
        return null;
    }

    public String getTextStringValue(Object text) {
        return ((Text)text).toString();
    }

    
    
    public boolean isAttribute(Object object) {
        return object instanceof Attribute;
    }

    public boolean isComment(Object object) {
        return object instanceof Comment;
    }

    public boolean isDocument(Object object) {
        return object instanceof Document;
    }

    public boolean isElement(Object object) {
        return object instanceof Element;
    }

    public boolean isNamespace(Object arg0) {
        return false;
    }

    public boolean isProcessingInstruction(Object arg0) {
        return false;
    }

    public boolean isText(Object object) {
        return object instanceof Text;
    }

    public XPath parseXPath(String xpath) throws org.jaxen.saxpath.SAXPathException {
        return new TapestryXPath(xpath);
    }

}