package com.formos.tapestry.xpath;

import java.util.List;

import org.apache.tapestry5.dom.Attribute;
import org.apache.tapestry5.dom.Element;
import org.apache.tapestry5.dom.Node;
import org.jaxen.BaseXPath;
import org.jaxen.FunctionContext;
import org.jaxen.JaxenException;
import org.jaxen.NamespaceContext;
import org.jaxen.Navigator;
import org.jaxen.VariableContext;
import org.jaxen.XPath;
import org.jaxen.expr.Expr;

/**
 * As {@link TapestryXPath} but throws unchecked exceptions.
 *
 * @author Paul Field
 */
public class UncheckedTapestryXPath implements XPath {
    private static final long serialVersionUID = 7040266295703699568L;

    private final TapestryXPath xpath;

    public UncheckedTapestryXPath(String xpath) {
        try {
            this.xpath = new TapestryXPath(xpath);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }


    /**
     *
     * @param xpath the XPath expression as a string (not null)
     * Creates a new TapestryXPath object.
     * @return the new instance (not null)
     */
    public static UncheckedTapestryXPath xpath(String xpath) {
        return new UncheckedTapestryXPath(xpath);
    }


    /**
     * @see XPath#addNamespace(String, String)
     */
    public void addNamespace(String prefix, String uri) {
        try {
            xpath.addNamespace(prefix, uri);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }


    /**
     * @see XPath#booleanValueOf(Object)
     */
    public boolean booleanValueOf(Object node) {
        try {
            return xpath.booleanValueOf(node);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }



    /**
     * @see BaseXPath#debug
     */
    public String debug() {
        return xpath.debug();
    }

    /**
     * @see XPath#evaluate(Object)
     */
    public Object evaluate(Object arg0) {
        try {
            return xpath.evaluate(arg0);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }


    /**
     * @see XPath#getFunctionContext()
     */
    public FunctionContext getFunctionContext() {
        return xpath.getFunctionContext();
    }


    /**
     * @see XPath#getNamespaceContext()
     */
    public NamespaceContext getNamespaceContext() {
        return xpath.getNamespaceContext();
    }


    /**
     * @see XPath#getNavigator()
     */
    public Navigator getNavigator() {
        return xpath.getNavigator();
    }


    /**
     * @see BaseXPath#getRootExpr()
     */
    public Expr getRootExpr() {
        return xpath.getRootExpr();
    }

    /**
     * @see XPath#getVariableContext()
     */
    public VariableContext getVariableContext() {
        return xpath.getVariableContext();
    }


    /**
     * @see XPath#numberValueOf(Object)
     */
    public Number numberValueOf(Object node) {
        try {
            return xpath.numberValueOf(node);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }


    /**
     * @see TapestryXPath#selectAttributes(Object)
     */
    public List<Attribute> selectAttributes(Object node) {
        try {
            return xpath.selectAttributes(node);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }


    /**
     * @see TapestryXPath#selectElements(Object)
     */
    public List<Element> selectElements(Object node) {
        try {
            return xpath.selectElements(node);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }


    /**
     * @see TapestryXPath#selectElementsAttribute(Object, String)
     */
    public List<String> selectElementsAttribute(Object node, String attributeName) {
        try {
            return xpath.selectElementsAttribute(node, attributeName);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }


    /**
     * @see TapestryXPath#selectElementsChildMarkup(Object)
     */
    public List<String> selectElementsChildMarkup(Object node) {
        try {
            return xpath.selectElementsChildMarkup(node);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }


    /**
     * @see XPath#selectNodes(Object)
     */
    public List<Node> selectNodes(Object node) {
        try {
            return xpath.selectNodes(node);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }


    /**
     * @see TapestryXPath#selectSingleElement(Object)
     */
    public Element selectSingleElement(Object node) {
        try {
            return xpath.selectSingleElement(node);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }


    /**
     * @see XPath#selectSingleNode(Object)
     */
    public Object selectSingleNode(Object node) {
        try {
            return xpath.selectSingleNode(node);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }


    /**
     * @see TapestryXPath#singleNormalizedDescendantText(Object)
     */
    public String singleNormalizedDescendantText(Object node) {
        try {
            return xpath.singleNormalizedDescendantText(node);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }


    /**
     * @see TapestryXPath#normalizedDescendantText(Object)
     */
    public List<String> normalizedDescendantText(Object node) {
        try {
            return xpath.normalizedDescendantText(node);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }


    /**
     * @see TapestryXPath#normalizeText(String)
     */
    public static String normalizeText(String string) {
        return TapestryXPath.normalizeText(string);
    }



    /**
     * @see XPath#setFunctionContext(FunctionContext)
     */
    public void setFunctionContext(FunctionContext functionContext) {
        xpath.setFunctionContext(functionContext);
    }


    /**
     * @see XPath#setNamespaceContext(NamespaceContext)
     */
    public void setNamespaceContext(NamespaceContext namespaceContext) {
        xpath.setNamespaceContext(namespaceContext);
    }


    /**
     * @see XPath#setVariableContext(VariableContext)
     */
    public void setVariableContext(VariableContext variableContext) {
        xpath.setVariableContext(variableContext);
    }


    /**
     * @see XPath#stringValueOf(Object)
     */
    public String stringValueOf(Object node) {
        try {
            return xpath.stringValueOf(node);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }


    /**
     * @see TapestryXPath#stringValuesOf(Object)
     */
    public List<String> stringValuesOf(Object node) {
        try {
            return xpath.stringValuesOf(node);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }


    @Deprecated
    public String valueOf(Object node) {
        try {
            return xpath.valueOf(node);
        } catch (JaxenException e) {
            throw new TapestryXPathException(e);
        }
    }
}